package PERPathPlanner;
import PER.rover.*;
import java.awt.image.BufferedImage;

import PER.rover.Action;
import PER.rover.ActionConstants;
import PER.rover.DriveToAction;
import PER.rover.Log;
import PER.rover.Rover;

public class PERController {
	public Rover rov;
    /** Thread of the currently executing command. */
    private Thread currThread;
    /** The full resolution version of the current image. */
    private BufferedImage fullImage;
    /** Version of the current image that is scaled to fit the display area. */
    private BufferedImage scaledImage;
    /** Set to true when the Stop button is hit. */
    private boolean stop;
    int robotX, robotY;
    double robotTheta; 
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		
	}
	private void outputln(String str) {
		System.out.println(str);
	}
	
	public boolean initCommunications(String IP) {
        boolean success;
        rov = new Rover();
        //initialize communications
        success = rov.initComm(IP);
        rov.setCurrentIP(IP); //saves the current ip in case communication needs to be reestablished
        rov.saveIP(IP); //save the IP address so it can be loaded the next time the program is run
        
        if (success) {
            success = rov.initRobot();
            success = rov.refresh();
            if (success) {
                outputln("Connected");
                return true;
            } else
                outputln("Error: rover is not responding! Please check that the rover is on and ready, and then try again.");
        } else
            outputln("Error: could not connect to the rover! Check that you have entered the IP address correctly. Also, make sure the rover is on and ready.");
        return false;
        
        
    }
	
	public void followPath(int[][] waypoints)  { 
		/*****************
		 * Insert your code here to have the PER follow the provided waypoints
		 ***************/
	}
	
	public boolean executeAction(Action a) {
		long last_update = 0;
		if(a == null)
            return false;
        
        Log.println("About to execute action: " + a.getSummary(), true);
        
        if (stop)
            return false;
        
        //start the action
        if (!a.doAction(rov))
            return false;
        
        //wait for the action to finish
        while (!stop && !a.isCompleted()) {
            try {
                Thread.sleep(10);
            } catch (java.lang.InterruptedException e) {
                outputln("executeAction interrupted");
            }
        }
        
        if (stop) {
            Log.println("Stopping the current action.");
            a.kill();
        }
        
        Log.println("Action completed (" + ActionConstants.getErrorText(a.getReturnValue()) + ").", true);
        return (!stop && a.isSuccess());
    }
}
