package PERPathPlanner;

import java.awt.geom.Line2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.*;

public class VisibilityGraph {
	/**
	 * Original obstacles specified in the source file
	 */
	public ArrayList origObstacles;

	/**
	 * Grown obstacles 
	 */
	public ArrayList gObstacles;

	/**
	 * Vertices graph
	 */
	public ArrayList vGraph;

	/**
	 * Shortest path computed from Visibility graph
	 */
	public ArrayList shortestPath;

	/**
	 * Starting point for robot
	 */
	public Vertex start;

	/**
	 * Goal of robot
	 */
	public Vertex goal;

	
	/**
	 * set of computed waypoints to be provided to PERController's followPath()
	 * to direct the PER from the starting point to the goal 
	 */
	public int[][] waypoints;

	public ArrayList getGrownObstacles() {
		return gObstacles;
	}
	
	public ArrayList getVisibilityGraph() {
		return vGraph;
	}
	
	public ArrayList getShortestPath() {
		return shortestPath;
	}

	/**
	 * Read in the start and goal vertices from the specified file.
	 * @param filename
	 */
	public void readStartAndGoal(String filename) {
		File inputFile;
		BufferedReader br;
		StringTokenizer st;

		try {
			inputFile = new File(filename);
			br = new BufferedReader(new FileReader(inputFile));

			st = new StringTokenizer(br.readLine());
			start = new Vertex(new Float(st.nextToken()).floatValue(),
					new Float(st.nextToken()).floatValue());

			st = new StringTokenizer(br.readLine());
			goal = new Vertex(new Float(st.nextToken()).floatValue(),
					new Float(st.nextToken()).floatValue());

			br.close();

		} catch (Exception e) {
			System.out.println("error: " + e.getMessage());
		}
	}
	
	/**
	 * Reads the obstacles from the specified text file
	 * @param filename
	 * @return
	 */
	public ArrayList readObstacles(String filename) {
		File inputFile;
		BufferedReader br;
		StringTokenizer st;
		ArrayList obstacles = new ArrayList();
		ArrayList vertices;
		int numObstacles, numVertices;
		Obstacle o;
		Vertex v;

		try {
			inputFile = new File(filename);
			br = new BufferedReader(new FileReader(inputFile));

			numObstacles = new Integer(br.readLine()).intValue();
			obstacles.ensureCapacity(numObstacles);

			for (int i = 0; i < numObstacles; i++) {
				o = new Obstacle();
				vertices = new ArrayList();

				numVertices = new Integer(br.readLine()).intValue();

				for (int j = 0; j < numVertices; j++) {
					st = new StringTokenizer(br.readLine());
					v = new Vertex(new Float(st.nextToken()).floatValue(),
							new Float(st.nextToken()).floatValue());
					vertices.add(v);
				}

				for (int j = 0; j < vertices.size(); j++) {
					if (j == vertices.size() - 1)
						o.addEdge(new Edge((Vertex) vertices.get(j),
								(Vertex) vertices.get(0)));
					else
						o.addEdge(new Edge((Vertex) vertices.get(j),
								(Vertex) vertices.get(j + 1)));
				}

				obstacles.add(o);
			}

			br.close();
		} catch (Exception e) {
			System.out.println("error: " + e.getMessage());
		}

		return obstacles;
	}
	
	/**
	 * Compute the visibility graph for the environment defined by obstaclesFile,
	 * with the start and goal defiend by startGoalFile
	 * @param obstaclesFile
	 * @param startGoalFile
	 */
	public void computeVisibilityGraph(String obstaclesFile,String startGoalFile) {
		//Read in obstacles and start/goal
		origObstacles = readObstacles(obstaclesFile);
		readStartAndGoal(startGoalFile);
		
		gObstacles = new ArrayList(); 
		gObstacles.add(origObstacles.get(0)); //first obstacle in original is the wall/boundary, add it to the grownObstacles as is
		
		/************************************
		 * Implement the below functionality
		 ************************************
		//calculate grown obstacles
		gObstacles = growObstacles(...);

		//calculate visibility graph
		vGraph = makeVisibilityGraph(...);
		
		//calculate shortest path
		shortestPath = DijkstrasAlgorithm.calculateShortestPath(vGraph,start,goal);
		
		//calculate waypoints for PER to follow (from shortestPath)
		waypoints = calculateWaypoints(...); 	
		*/
	}
}